package Device

import (
	"Cold_Data/logs"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"time"
)

// 模板
type DeviceParameter struct {
	Id int `orm:"column(ID);size(11);auto;pk"`

	T_sn string `orm:"size(256);"` // 设备序列号 KF开头，环境监测主机。 YD开头，温途监测主机

	T_name     string `orm:"size(256);"` //设备名称 （传 * 代表托管给平台处理） *
	T_saveT    int    `orm:"size(5);"`   //数据保存间隔 s(60~3600) 默认：60 *
	T_overA    int    `orm:"size(5);"`   //数据保存间隔 超限报警触发间隔 s(60~600) 默认：60  *
	T_outA     int    `orm:"size(5);"`   //超限报警触发间隔 s(0,60~600)  为0时只触发一次 默认：60 *
	T_lostA    int    `orm:"size(5);"`   //传感器掉线报警触发间隔  s(0,60~600) 为0时只触发一次 默认：60  *
	T_bat      int    `orm:"size(5);"`   // 电池电量下限   %(0~30) 默认：20 *
	T_warn     int    `orm:"size(5);"`   // 超限预警触发间隔 s(0,60~600)为0时只触发一次 默认：60  *
	T_warnD    int    `orm:"size(5);"`   // 超限预警延时   s(0~600) 默认：0 *
	T_scan     int    `orm:"size(5);"`   // 显示轮播间隔   s(1~240) 默认：5 *
	T_dormancy int    `orm:"size(5);"`   // 是否进入休眠  0:关闭 1:开启    默认：0
	T_snum     int    `orm:"size(5);"`   // 【管理主机】 - 传感器数量  (范围0~255)
	T_btname   string `orm:"size(256);"` //蓝牙打印机名称
	T_btsid    int    `orm:"size(256);"` //打印机服务号
	T_btchar   int    `orm:"size(256);"` //蓝牙特征码

	T_uuid      string `orm:"size(256);"`          //处理 人员
	T_Msid      int64  `orm:"size(50);default(0)"` // 消息识别ID
	T_SendState int    `orm:"size(2);default(1)"`  // 发送状态  0 待发送   1 发送成功  2 发送失败  3 丢弃
	T_State     int    `orm:"size(2);default(1)"`  //  1 系统获取   2 用户提交

	CreateTime time.Time `orm:"column(create_time);type(timestamp);auto_now_add"` //auto_now_add 第一次保存时才设置时间
	UpdateTime time.Time `orm:"column(update_time);type(timestamp);auto_now"`     //auto_now 每次 model 保存时都会对时间自动更新
}

func (t *DeviceParameter) TableName() string {
	return "device_parameter" // 数据库名称   // ************** 替换 FormulaList **************
}

func init() {
	//注册模型
	orm.RegisterModel(new(DeviceParameter))
	orm.Debug = true
}

// ---------------- 特殊方法 -------------------
// 添加
func Add_DeviceParameter_fun(Deviceparameter DeviceParameter) bool {
	if DeviceParameter_r, is := Read_DeviceParameter(Deviceparameter.T_sn); is {
		if Deviceparameter.T_name != DeviceParameter_r.T_name ||
			Deviceparameter.T_saveT != DeviceParameter_r.T_saveT ||
			Deviceparameter.T_overA != DeviceParameter_r.T_overA ||
			Deviceparameter.T_outA != DeviceParameter_r.T_outA ||
			Deviceparameter.T_lostA != DeviceParameter_r.T_lostA ||
			Deviceparameter.T_bat != DeviceParameter_r.T_bat ||
			Deviceparameter.T_scan != DeviceParameter_r.T_scan ||
			Deviceparameter.T_warn != DeviceParameter_r.T_warn ||
			Deviceparameter.T_warnD != DeviceParameter_r.T_warnD ||
			Deviceparameter.T_dormancy != DeviceParameter_r.T_dormancy ||
			Deviceparameter.T_snum != DeviceParameter_r.T_snum ||
			Deviceparameter.T_btname != DeviceParameter_r.T_btname ||
			//Deviceparameter.T_speed != DeviceParameter_r.T_speed ||
			Deviceparameter.T_btsid != DeviceParameter_r.T_btsid ||
			Deviceparameter.T_btchar != DeviceParameter_r.T_btchar {
			if _, is := Add_DeviceParameter(Deviceparameter); !is {
				logs.PrintlnError("Add_DeviceParameter_fun", "插入失败 [DeviceParameter]")
			}
			return true
		}
	} else {
		if _, is := Add_DeviceParameter(Deviceparameter); !is {
			logs.PrintlnError("Add_DeviceParameter_fun", "插入失败 [DeviceParameter]")
		}
		return true
	}
	return false
}

// 添加
func Add_DeviceParameter(m DeviceParameter) (DeviceParameter, bool) {
	o := orm.NewOrm()
	m.Id = 0
	id, err := o.Insert(&m)
	if err != nil {
		logs.Println("Add_DeviceParameter", err)
		return m, false
	}
	m.Id = int(id)
	return m, true
}

// 获取设备参数
func Read_DeviceParameter(T_sn string) (t DeviceParameter, bool bool) {

	o := orm.NewOrm()
	qs := o.QueryTable(new(DeviceParameter))

	cond := orm.NewCondition()
	cond = cond.And("T_sn", T_sn).And("T_State", 1)

	qs.SetCond((*orm2.Condition)(cond)).OrderBy("-UpdateTime").One(&t)
	// 判断是否有数据
	if t.Id == 0 {
		return t, false
	}

	return t, true
}

// 修改
func Update_DeviceParameter(r DeviceParameter, cols ...string) bool {
	o := orm.NewOrm()
	if num, err := o.Update(&r, cols...); err == nil {
		logs.Println("Number of records updated in database:", num)
		return true
	}
	return false
}

// 获取列表
func Read_DeviceParameter_SN_Msid(T_sn string, T_Msid int64) (r DeviceParameter, err error) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名

	qs := o.QueryTable(new(DeviceParameter))

	err = qs.Filter("T_sn", T_sn).Filter("T_Msid", T_Msid).One(&r)
	if err != nil {
		logs.Println("Read_DeviceParameter_SN_Msid:", err)
	}
	return r, err
}

// 修改
func Update_DeviceParameter_(m DeviceParameter) (err error) {
	o := orm.NewOrm()
	if num, err := o.Update(&m, "T_SendState"); err == nil {
		logs.Println("Number of records updated in database:", num)
	}

	return err
}

// 获取列表
func Read_DeviceParameter_SN_T_State_1(T_sn string) (r []DeviceParameter) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名

	qs := o.QueryTable(new(DeviceParameter))

	qs.Limit(3, 0).Filter("T_sn", T_sn).Filter("T_State", 2).OrderBy("-Id").All(&r)

	return r
}

// 获取列表
func Read_DeviceParameter_SN_T_SendState_0(T_sn string) (r []DeviceParameter) {

	o := orm.NewOrm()
	// 也可以直接使用 Model 结构体作为表名

	qs := o.QueryTable(new(DeviceParameter))

	qs.Limit(1, 0).Filter("T_sn", T_sn).Filter("T_State", 2).Filter("T_SendState", 0).OrderBy("-Id").All(&r)

	return r
}

// 获取最新数据
func UPDATE_DeviceParameter_SN_T_SendState_2_sql(T_sn string) {
	o := orm.NewOrm()

	res, err := o.Raw("UPDATE device_parameter SET `t__send_state` = 3 WHERE `t_sn` LIKE '%" + T_sn + "%' AND `t__send_state` = '0' AND `t__state` = '2'").Exec()
	if err == nil {
		num, _ := res.RowsAffected()
		logs.Println("mysql row affected nums: ", num)
	}

}
